@@ -1,6 +1,6 @@
 /*
   Racket
-  Copyright (c) 2004-2011 PLT Scheme Inc.
+  Copyright (c) 2004-2013 PLT Design Inc.
   Copyright (c) 1995-2001 Matthew Flatt
   All rights reserved.
 
@@ -63,6 +63,39 @@
 # define MZ_USE_SINGLE_FLOATS
 #endif
 
+/* gcc defines __SSE_MATH__ when SSE floating point is enabled: */
+#ifdef __SSE_MATH__
+# define C_COMPILER_USES_SSE 1
+#endif
+
+#ifdef C_COMPILER_USES_SSE
+# if defined(MZ_TRY_EXTFLONUMS) && !defined(MZ_NO_EXTFLONUMS)
+#  define MZ_LONG_DOUBLE
+#  ifdef ASM_DBLPREC_CONTROL_87
+#   define ASM_EXTPREC_CONTROL_87
+#  endif
+# endif
+# ifdef ASM_DBLPREC_CONTROL_87
+#  undef ASM_DBLPREC_CONTROL_87
+# endif
+# if defined(MZ_USE_JIT_I386) && !defined(MZ_NO_JIT_SSE)
+#  define MZ_USE_JIT_SSE
+# endif
+#endif
+
+#ifdef MZ_LONG_DOUBLE
+# if defined(_MSC_VER)
+#  define BYTES_RESERVED_FOR_LONG_DOUBLE 16
+typedef struct {
+  char bytes[BYTES_RESERVED_FOR_LONG_DOUBLE];
+} mz_long_double;
+# else
+typedef long double mz_long_double;
+# endif
+#else
+typedef double mz_long_double;
+#endif
+
 #ifdef DONT_ITIMER
 # undef USE_ITIMER
 #endif
@@ -110,6 +143,9 @@ typedef long FILE;
 #ifndef MSC_IZE
 # define MSC_IZE(x) x
 #endif
+#ifndef M_MSC_IZE
+# define M_MSC_IZE(x) x
+#endif
 #ifndef MSCBOR_IZE
 # define MSCBOR_IZE(x) MSC_IZE(x)
 #endif
@@ -127,10 +163,14 @@ typedef long FILE;
    old way. */
 #ifdef MZ_USE_OLD_ARRAY_STYLE
 # define mzFLEX_ARRAY_DECL 1
+# define mzFLEX_ARRAY4_DECL 4
 # define mzFLEX_DELTA 1
+# define mzFLEX4_DELTA 4
 #else
 # define mzFLEX_ARRAY_DECL /* empty */
+# define mzFLEX_ARRAY4_DECL /* empty */
 # define mzFLEX_DELTA 0
+# define mzFLEX4_DELTA 0
 #endif
 
 #ifdef MZ_XFORM
@@ -237,8 +277,8 @@ typedef _int64 mzlonglong;
 typedef unsigned _int64 umzlonglong;
 #else
 # if defined(NO_LONG_LONG_TYPE) || defined(SIXTY_FOUR_BIT_INTEGERS)
-typedef long mzlonglong;
-typedef unsigned long umzlonglong;
+typedef intptr_t mzlonglong;
+typedef uintptr_t umzlonglong;
 # else
 typedef long long mzlonglong;
 typedef unsigned long long umzlonglong;
@@ -321,6 +361,18 @@ typedef struct {
   double double_val;
 } Scheme_Double;
 
+#ifdef MZ_LONG_DOUBLE
+typedef struct {
+  Scheme_Object so;
+  mz_long_double long_double_val;
+} Scheme_Long_Double;
+#else
+typedef struct {
+  Scheme_Object so;
+  const char *printed_form;
+} Scheme_Long_Double;
+#endif 
+
 #ifdef MZ_USE_SINGLE_FLOATS
 typedef struct {
   Scheme_Object so;
@@ -331,7 +383,7 @@ typedef struct {
 typedef struct Scheme_Symbol {
   Scheme_Inclhash_Object iso; /* 1 in low bit of keyex indicates uninterned */
   intptr_t len;
-  char s[4]; /* Really, a number of chars to match `len' */
+  char s[mzFLEX_ARRAY4_DECL];
 } Scheme_Symbol;
 
 typedef struct Scheme_Vector {
@@ -350,6 +402,14 @@ typedef struct Scheme_Double_Vector {
   double els[mzFLEX_ARRAY_DECL];
 } Scheme_Double_Vector;
 
+#ifdef MZ_LONG_DOUBLE
+typedef struct Scheme_Long_Double_Vector {
+  Scheme_Inclhash_Object iso; /* & 0x2 indicates allocated in the MASTERGC */
+  intptr_t size;
+  mz_long_double els[mzFLEX_ARRAY_DECL];
+} Scheme_Long_Double_Vector;
+#endif
+
 typedef struct Scheme_Print_Params Scheme_Print_Params;
 typedef void (*Scheme_Type_Printer)(Scheme_Object *v, int for_display, Scheme_Print_Params *pp);
 
@@ -409,6 +469,8 @@ typedef intptr_t (*Scheme_Secondary_Hash_Proc)(Scheme_Object *obj, void *cycle_d
 #define SCHEME_REALP(obj)  (SCHEME_INTP(obj) || ((_SCHEME_TYPE(obj) >= scheme_bignum_type) && (_SCHEME_TYPE(obj) < scheme_complex_type)))
 #define SCHEME_NUMBERP(obj)  (SCHEME_INTP(obj) || ((_SCHEME_TYPE(obj) >= scheme_bignum_type) && (_SCHEME_TYPE(obj) <= scheme_complex_type)))
 
+#define SCHEME_LONG_DBLP(obj)     SAME_TYPE(SCHEME_TYPE(obj), scheme_long_double_type)
+
 #define SCHEME_CHAR_STRINGP(obj)  SAME_TYPE(SCHEME_TYPE(obj), scheme_char_string_type)
 #define SCHEME_MUTABLE_CHAR_STRINGP(obj)  (SCHEME_CHAR_STRINGP(obj) && SCHEME_MUTABLEP(obj))
 #define SCHEME_IMMUTABLE_CHAR_STRINGP(obj)  (SCHEME_CHAR_STRINGP(obj) && SCHEME_IMMUTABLEP(obj))
@@ -450,6 +512,9 @@ typedef intptr_t (*Scheme_Secondary_Hash_Proc)(Scheme_Object *obj, void *cycle_d
 #define SCHEME_MUTABLE_BOXP(obj)  (SCHEME_BOXP(obj) && SCHEME_MUTABLEP(obj))
 #define SCHEME_IMMUTABLE_BOXP(obj)  (SCHEME_BOXP(obj) && SCHEME_IMMUTABLEP(obj))
 
+#define SCHEME_PROMPT_TAGP(obj) SAME_TYPE(SCHEME_TYPE(obj), scheme_prompt_tag_type)
+#define SCHEME_CONTINUATION_MARK_KEYP(obj) SAME_TYPE(SCHEME_TYPE(obj), scheme_continuation_mark_key_type)
+
 #define SCHEME_BUCKTP(obj) SAME_TYPE(SCHEME_TYPE(obj),scheme_bucket_table_type)
 #define SCHEME_HASHTP(obj) SAME_TYPE(SCHEME_TYPE(obj),scheme_hash_table_type)
 #define SCHEME_HASHTRP(obj) SAME_TYPE(SCHEME_TYPE(obj),scheme_hash_tree_type)
@@ -459,6 +524,7 @@ typedef intptr_t (*Scheme_Secondary_Hash_Proc)(Scheme_Object *obj, void *cycle_d
 #define SCHEME_IMMUTABLE_VECTORP(obj)  (SCHEME_VECTORP(obj) && SCHEME_IMMUTABLEP(obj))
 
 #define SCHEME_FLVECTORP(obj)  SAME_TYPE(SCHEME_TYPE(obj), scheme_flvector_type)
+#define SCHEME_EXTFLVECTORP(obj)  SAME_TYPE(SCHEME_TYPE(obj), scheme_extflvector_type)
 #define SCHEME_FXVECTORP(obj)  SAME_TYPE(SCHEME_TYPE(obj), scheme_fxvector_type)
 
 #define SCHEME_STRUCTP(obj) (SAME_TYPE(SCHEME_TYPE(obj), scheme_structure_type) || SAME_TYPE(SCHEME_TYPE(obj), scheme_proc_struct_type))
@@ -525,6 +591,9 @@ typedef intptr_t (*Scheme_Secondary_Hash_Proc)(Scheme_Object *obj, void *cycle_d
 #define SCHEME_CHAR_VAL(obj) (((Scheme_Small_Object *)(obj))->u.char_val)
 #define SCHEME_INT_VAL(obj)  (OBJ_TO_LONG(obj)>>1)
 #define SCHEME_DBL_VAL(obj)  (((Scheme_Double *)(obj))->double_val)
+#ifdef MZ_LONG_DOUBLE
+#define SCHEME_LONG_DBL_VAL(obj)  (((Scheme_Long_Double *)(obj))->long_double_val)
+#endif
 #ifdef MZ_USE_SINGLE_FLOATS
 # define SCHEME_FLT_VAL(obj)  (((Scheme_Float *)(obj))->float_val)
 # define SCHEME_FLOAT_VAL(obj) (SCHEME_DBLP(obj) ? SCHEME_DBL_VAL(obj) : SCHEME_FLT_VAL(obj))
@@ -571,6 +640,11 @@ typedef intptr_t (*Scheme_Secondary_Hash_Proc)(Scheme_Object *obj, void *cycle_d
 #define SCHEME_FLVEC_SIZE(obj) (((Scheme_Double_Vector *)(obj))->size)
 #define SCHEME_FLVEC_ELS(obj)  (((Scheme_Double_Vector *)(obj))->els)
 
+#ifdef MZ_LONG_DOUBLE
+#define SCHEME_EXTFLVEC_SIZE(obj) (((Scheme_Long_Double_Vector *)(obj))->size)
+#define SCHEME_EXTFLVEC_ELS(obj)  (((Scheme_Long_Double_Vector *)(obj))->els)
+#endif
+
 #define SCHEME_FXVEC_SIZE(obj) SCHEME_VEC_SIZE(obj)
 #define SCHEME_FXVEC_ELS(obj) SCHEME_VEC_ELS(obj)
 
@@ -657,16 +731,14 @@ typedef struct Scheme_Offset_Cptr
    Do not use them directly. */
 #define SCHEME_PRIM_OPT_MASK (1 | 2)
 #define SCHEME_PRIM_IS_PRIMITIVE 4
-#define SCHEME_PRIM_IS_UNSAFE_OMITABLE 8
-#define SCHEME_PRIM_IS_STRUCT_OTHER 16
+#define SCHEME_PRIM_IS_MULTI_RESULT 8
+#define SCHEME_PRIM_IS_CLOSURE 16
 #define SCHEME_PRIM_OTHER_TYPE_MASK (32 | 64 | 128 | 256)
-#define SCHEME_PRIM_IS_MULTI_RESULT 512
-#define SCHEME_PRIM_IS_BINARY_INLINED 1024
-#define SCHEME_PRIM_IS_UNSAFE_FUNCTIONAL 2048
-#define SCHEME_PRIM_IS_METHOD 4096
-#define SCHEME_PRIM_IS_CLOSURE 8192
-#define SCHEME_PRIM_IS_UNARY_INLINED 16384
-#define SCHEME_PRIM_IS_NARY_INLINED 32768
+#define SCHEME_PRIM_IS_METHOD 512
+
+#define SCHEME_PRIM_OPT_INDEX_SIZE 6
+#define SCHEME_PRIM_OPT_INDEX_SHIFT 10
+#define SCHEME_PRIM_OPT_INDEX_MASK ((1 << SCHEME_PRIM_OPT_INDEX_SIZE) - 1)
 
 /* Values with SCHEME_PRIM_OPT_MASK, earlier implies later: */
 #define SCHEME_PRIM_OPT_FOLDING    3
@@ -674,10 +746,12 @@ typedef struct Scheme_Offset_Cptr
 #define SCHEME_PRIM_OPT_NONCM      1
 
 /* Values with SCHEME_PRIM_OTHER_TYPE_MASK */
-#define SCHEME_PRIM_STRUCT_TYPE_INDEXLESS_GETTER 0
+#define SCHEME_PRIM_STRUCT_TYPE_INDEXLESS_GETTER (32 | 256)
 #define SCHEME_PRIM_STRUCT_TYPE_CONSTR           128
+#define SCHEME_PRIM_STRUCT_TYPE_SIMPLE_CONSTR    (32 | 64 | 128)
 #define SCHEME_PRIM_STRUCT_TYPE_INDEXLESS_SETTER 256
 #define SCHEME_PRIM_STRUCT_TYPE_INDEXED_SETTER   (128 | 256)
+#define SCHEME_PRIM_STRUCT_TYPE_BROKEN_INDEXED_SETTER   (32 | 128)
 #define SCHEME_PRIM_TYPE_PARAMETER               64
 #define SCHEME_PRIM_TYPE_STRUCT_PROP_GETTER      (64 | 128)
 #define SCHEME_PRIM_SOMETIMES_INLINED            (64 | 256)
@@ -901,7 +975,7 @@ typedef struct {
 #endif
 
 /* Like setjmp & longjmp, but you can jmp to a deeper stack position */
-/* Intialize a Scheme_Jumpup_Buf record before using it */
+/* Initialize a Scheme_Jumpup_Buf record before using it */
 typedef struct Scheme_Jumpup_Buf {
   void *stack_from, *stack_copy;
   intptr_t stack_size, stack_max_size;
@@ -1043,6 +1117,8 @@ typedef struct Scheme_Thread {
   Scheme_Object *dead_box;      /* contains non-zero when the thread is dead */
   Scheme_Object *running_box;   /* contains pointer to thread when it's running */
 
+  struct Scheme_Thread *gc_prep_chain;
+
   struct Scheme_Thread *nester, *nestee;
 
   struct future_t *current_ft;
@@ -1070,7 +1146,7 @@ typedef struct Scheme_Thread {
 
   struct Scheme_Marshal_Tables *current_mt;
 
-  Scheme_Object *constant_folding; /* compiler hack */
+  struct Optimize_Info *constant_folding; /* compiler hack */
   Scheme_Object *reading_delayed; /* reader hack */
 
   Scheme_Object *(*overflow_k)(void);
@@ -1144,9 +1220,13 @@ typedef struct Scheme_Thread {
 
   long saved_errno;
 
+  int futures_slow_path_tracing;
+
 #ifdef MZ_PRECISE_GC
   struct GC_Thread_Info *gc_info; /* managed by the GC */
+  void *place_channel_msg_in_flight;
 #endif
+
 } Scheme_Thread;
 
 #include "schthread.h"
@@ -1271,6 +1351,7 @@ enum {
   MZCONFIG_CODE_INSPECTOR,
 
   MZCONFIG_USE_COMPILED_KIND,
+  MZCONFIG_USE_COMPILED_ROOTS,
   MZCONFIG_USE_USER_PATHS,
   MZCONFIG_USE_LINK_PATHS,
 
@@ -1369,13 +1450,16 @@ struct Scheme_Port
   Scheme_Location_Fun location_fun;
   Scheme_Count_Lines_Fun count_lines_fun;
   Scheme_Buffer_Mode_Fun buffer_mode_fun;
+  Scheme_Object *position_redirect; /* for `file-position' */
 };
 
 struct Scheme_Input_Port
 {
   struct Scheme_Port p;
+  char slow; /* 0 => no line count, no ungotten, etc.: can call get_string_fun directly */
   char closed, pending_eof;
   Scheme_Object *sub_type;
+  Scheme_Object *closed_evt;
   Scheme_Custodian_Reference *mref;
   void *port_data;
   Scheme_Get_String_Fun get_string_fun;
@@ -1401,6 +1485,7 @@ struct Scheme_Output_Port
   struct Scheme_Port p;
   short closed;
   Scheme_Object *sub_type;
+  Scheme_Object *closed_evt;
   Scheme_Custodian_Reference *mref;
   void *port_data;
   Scheme_Write_String_Evt_Fun write_string_evt_fun;
@@ -1726,8 +1811,6 @@ extern void *scheme_malloc_envunbox(size_t);
 /*                   embedding configuration and hooks                    */
 /*========================================================================*/
 
-typedef void (*Scheme_On_Atomic_Timeout_Proc)(int must_give_up);
-
 #if SCHEME_DIRECT_EMBEDDED
 
 #if defined(_IBMR2)
@@ -1757,6 +1840,7 @@ MZ_EXTERN void scheme_set_startup_load_on_demand(int);
 MZ_EXTERN void scheme_set_ignore_user_paths(int);
 MZ_EXTERN void scheme_set_ignore_link_paths(int);
 MZ_EXTERN void scheme_set_logging(int syslog_level, int stderr_level);
+MZ_EXTERN void scheme_set_logging_spec(Scheme_Object *syslog_level, Scheme_Object *stderr_level);
 
 MZ_EXTERN int scheme_get_allow_set_undefined();
 
@@ -1769,6 +1853,8 @@ XFORM_NONGCING MZ_EXTERN intptr_t scheme_get_multiple_count();
 XFORM_NONGCING MZ_EXTERN Scheme_Object **scheme_get_multiple_array();
 XFORM_NONGCING MZ_EXTERN void scheme_set_current_thread_ran_some();
 
+MZ_EXTERN void scheme_embedded_load(intptr_t len, const char *s, int predefined);
+MZ_EXTERN void scheme_register_embedded_load(intptr_t len, const char *s);
 
 /* Set these global hooks (optionally): */
 typedef void (*Scheme_Exit_Proc)(int v);
@@ -1821,9 +1907,11 @@ MZ_EXTERN void scheme_set_addon_dir(Scheme_Object *p);
 MZ_EXTERN void scheme_set_links_file(Scheme_Object *p);
 MZ_EXTERN void scheme_set_command_line_arguments(Scheme_Object *vec);
 MZ_EXTERN void scheme_set_compiled_file_paths(Scheme_Object *list);
+MZ_EXTERN void scheme_set_compiled_file_roots(Scheme_Object *list);
 
 MZ_EXTERN void scheme_init_collection_paths(Scheme_Env *global_env, Scheme_Object *extra_dirs);
 MZ_EXTERN void scheme_init_collection_paths_post(Scheme_Env *global_env, Scheme_Object *extra_dirs, Scheme_Object *extra_post_dirs);
+MZ_EXTERN void scheme_init_compiled_roots(Scheme_Env *global_env, const char *paths);
 
 MZ_EXTERN void scheme_seal_parameters();
 
@@ -1866,8 +1954,6 @@ MZ_EXTERN void scheme_register_static(void *ptr, intptr_t size);
 # define MZ_REGISTER_STATIC(x) /* empty */
 #endif
 
-MZ_EXTERN Scheme_On_Atomic_Timeout_Proc scheme_on_atomic_timeout;
-
 MZ_EXTERN void scheme_immediate_exit(int status);
 
 MZ_EXTERN int scheme_new_param(void);
@@ -1884,7 +1970,7 @@ MZ_EXTERN Scheme_Object *scheme_register_parameter(Scheme_Prim *function, char *
 /*                              addrinfo                                  */
 /*========================================================================*/
 
-#ifdef HAVE_GETADDRINFO
+#if defined(HAVE_GETADDRINFO) || defined(__MINGW32__)
 # define mz_addrinfo addrinfo
 #else
 struct mz_addrinfo {
@@ -1985,6 +2071,9 @@ extern Scheme_Extension_Table *scheme_extension_table;
 #ifdef USE_BEOS_PORT_THREADS
 # define USE_FAR_MZ_FDCALLS
 #endif
+#ifdef HAVE_POLL_SYSCALL
+# define USE_FAR_MZ_FDCALLS
+#endif
 
 #ifdef USE_FAR_MZ_FDCALLS
 # define MZ_GET_FDSET(p, n) scheme_get_fdset(p, n)
@@ -1994,12 +2083,19 @@ extern Scheme_Extension_Table *scheme_extension_table;
 # define MZ_FD_ISSET(n, p) scheme_fdisset(p, n)
 #else
 # define MZ_GET_FDSET(p, n) ((void *)(((fd_set *)p) + n))
-# define MZ_FD_ZERO(p) FD_ZERO(p)
-# define MZ_FD_SET(n, p) FD_SET(n, p)
-# define MZ_FD_CLR(n, p) FD_CLR(n, p)
-# define MZ_FD_ISSET(n, p) FD_ISSET(n, p)
+# define MZ_FD_ZERO(p) FD_ZERO((fd_set *)(p))
+# define MZ_FD_SET(n, p) FD_SET(n, (fd_set *)(p))
+# define MZ_FD_CLR(n, p) FD_CLR(n, (fd_set *)(p))
+# define MZ_FD_ISSET(n, p) FD_ISSET(n, (fd_set *)(p))
 #endif
 
+/* For scheme_fd_to_semaphore(): */
+#define MZFD_CREATE_READ  1
+#define MZFD_CREATE_WRITE 2
+#define MZFD_CHECK_READ   3
+#define MZFD_CHECK_WRITE  4
+#define MZFD_REMOVE       5
+
 /*========================================================================*/
 
 #ifdef __cplusplus
